#ifndef BOHA_BULLET_INFO
#define BOHA_BULLET_INFO

#include <string>
#include <vector>
#include <algorithm>
#include <memory>
#include <map>

class Surface;

class BulletDescription {
public:
	explicit BulletDescription(std::istream& is);
	BulletDescription(const std::string& file, const std::string& title,
					  int id);

	int id() const { return id_; }
	const std::string& file() const { return file_; }
	const std::string& title() const { return title_; }
	const std::string& description() const { return description_; }
	const std::string& capture() const { return capture_; }
	std::string xmlFile() const;
	bool isUserDefined() const { return userDefined_; }

	void loadUserInfo(const std::string& userName);
	void saveUserInfo(const std::string& userName) const;
	void doCome() { come_++; }
	void doDamage() { damage_++; }
	void updateLimit(int limit) { limit_ = std::max(limit, limit_); }
	int getCome() const { return come_; }
	int getDamage() const { return damage_; }
	int getSuccessPer() const {
		if (come_ == 0) return 0;
		return (come_-damage_)*100/come_;
	}
	int getLimit() const { return limit_; }
	Surface* shot() { return shot_.get(); }
	bool hasShot() const { return hasShot_; }

	bool isIdLess(const BulletDescription* rhs) const {
		return id_ < rhs->id_;
	}

#ifdef ZAURUS
	void setScore(int score) { score_ = score; }
	int score() const { return score_; }
#endif

private:
	int id_;
	std::string file_, title_, description_, capture_;
	std::auto_ptr<Surface> shot_;
	bool hasShot_;

	int come_, damage_, limit_;

	bool userDefined_;

#ifdef ZAURUS
	int score_;
#endif

};

class BulletInfo {
public:
	typedef std::vector<BulletDescription*> Descriptions;

	explicit BulletInfo(const std::string& str);
	~BulletInfo();

	const Descriptions& descriptionsDisplayOrder() const {
		return descriptionsDisplayOrder_;
	}
	const Descriptions& descriptions() const {
		return descriptionsIdOrder_;
	}

	void loadUserInfo(const std::string& userName);
	void saveUserInfo() const;

	int getBulletNum() const { return officialBulletNum_; }

#ifdef ZAURUS
	void saveScore() const;
#endif

private:
	Descriptions descriptionsDisplayOrder_;
	Descriptions descriptionsIdOrder_;

	std::string userName_;

	int officialBulletNum_;

};

#endif // ! BOHA_BULLET_INFO
