#include "command.h"
#include "game.h"
#include "util.h"
#include "player.h"
#include "conf.h"
#include "replay.h"

#include <bulletml/bulletmlparser.h>

EnemyCommand::EnemyCommand(BulletMLParser* bulletml, Enemy* enemy)
    : BulletMLRunner(bulletml), enemy_(enemy)
{}

EnemyCommand::EnemyCommand(BulletMLState* state, Enemy* enemy)
    : BulletMLRunner(state), enemy_(enemy)
{}

BMLNum EnemyCommand::getBulletDirection() {
	return rtod(enemy_->spd().angle());
}

BMLNum EnemyCommand::getAimDirection() {
	return rtod(Game::instance()->player()->center().angle(enemy_->center()));
}

BMLNum EnemyCommand::getBulletSpeed() {
	static const Num_ r(0.01);
	return enemy_->spd().nlength() * r;
}

BMLNum EnemyCommand::getDefaultSpeed() {
	return BMLNum(1);
}

BMLNum EnemyCommand::getRank() {
	return BMLNum(Game::instance()->getRank());
}

void EnemyCommand::createSimpleBullet(BMLNum direction, BMLNum speed) {
	int d = Conf::instance()->density();
	if (d != 100) {
		if (rnd(100) >= d) return;
	}

	Game::instance()->addShot(enemy_->center(),
							  Point_(PolarCoord(speed*100, dtor(direction))),
							  enemy_);
}

void EnemyCommand::createBullet(BulletMLState* state,
								BMLNum direction, BMLNum speed) {
	int d = Conf::instance()->density();
	if (d != 100) {
		if (rnd(100) >= d) return;
	}

	Game::instance()->addEnemy(enemy_->center(),
							   Point_(PolarCoord(speed*100, dtor(direction))),
							   state, enemy_);
}

int EnemyCommand::getTurn() {
	return static_cast<int>(Game::instance()->getFrameTurn());
}

void EnemyCommand::doVanish() {
	enemy_->alive_ = false;
}

void EnemyCommand::doChangeDirection(BMLNum direction) {
	PolarCoord pc;
	pc.l = enemy_->spd().nlength();
	// ۑ邽߂̃AhzbNȉ
#ifndef ZAURUS
	if (pc.l == 0.0) pc.l = 0.001;
#else
	if (pc.l == 0.0) pc.l.val = 20;
#endif
	pc.r = dtor(direction);
#ifndef ZAURUS
	enemy_->spd_ = Point(pc);
#else
	Point_ tmp(pc);
	enemy_->spd_.x = Num(tmp.x);
	enemy_->spd_.y = Num(tmp.y);
#endif
}

void EnemyCommand::doChangeSpeed(BMLNum speed) {
	PolarCoord  pc(enemy_->spd());
	pc.l = speed * 100.0;
#ifndef ZAURUS
	enemy_->spd_ = Point(pc);
#else
	Point_ tmp(pc);
	enemy_->spd_.x = Num(tmp.x);
	enemy_->spd_.y = Num(tmp.y);
#endif
}

#ifndef ZAURUS
#define GET_BMLNUM(arg) arg
typedef BMLNum Num_;
#else
#define GET_BMLNUM(arg) arg##.getBMLNum()
typedef Num Num_;
#endif

void EnemyCommand::doAccelX(BMLNum speedx) {
	enemy_->spd_.x = speedx * 100;
}

void EnemyCommand::doAccelY(BMLNum speedy) {
	enemy_->spd_.y = speedy * 100;
}

BMLNum EnemyCommand::getBulletSpeedX() {
	static const BMLNum r(0.01);
	return enemy_->spd_.x * r;
}

BMLNum EnemyCommand::getBulletSpeedY() {
	static const BMLNum r(0.01);
	return enemy_->spd_.y * r;
}

BMLNum EnemyCommand::getRand() {
#ifndef ZAURUS
	return rep_rec_rand();
#else
	// adhoc! RAND_MAX must be 2^31, BMLNum must be Num<10>
	BMLNum ret;
	ret.val = rand();
	ret.val >>= 21;
	return ret;
#endif
}
