#include "game.h"
#include "conf.h"
#include "screen.h"

#include <bulletml/bulletmlerror.h>

#include <stdexcept>
#include <iostream>

#include <SDL.h>

#ifdef ZAURUS
#include <sys/types.h>
#include <signal.h>
#endif

void displayBulletMLError(const std::string& msg) {
#ifndef ZAURUS
	Screen* scr = Game::instance()->screen();

	scr->fill(Color::BLACK);

	scr->putString("BulletMLG[܂B",
				   Point(0, 50), Color::WHITE);
	scr->putString(msg.substr(0, msg.find(':')+1),
				   Point(0, 100), Color::WHITE);
	scr->putString(msg.substr(msg.find(':')+1),
				   Point(0, 150), Color::WHITE);
	scr->putString("{^ƏI܂B",
				   Point(0, 200), Color::WHITE);
	scr->putString("Ȃ̃G[ɐS肪΁A",
				   Point(0, 250), Color::WHITE);
	scr->putString("҂ɘAĒƍKłB",
				   Point(0, 300), Color::WHITE);
	scr->putString("  shinichiro.h (g940455@mail.ecc.u-tokyo.ac.jp)",
				   Point(0, 350), Color::WHITE);

	scr->flip();

	SDL_Delay(1000);

	Game::instance()->waitInputRelease();
	Game::instance()->waitAnyInput();
#endif
}

int main(int argc, char* argv[]) {
	Game* game = new Game;

	try {
		new Conf(argc, argv);

		game->init();

		int ret = 0;
#ifdef ZAURUS
		ret = game->title();
#endif
		while (ret == 0) {
#ifndef ZAURUS
			ret = game->title();
#endif
			if (0 == ret) ret = game->start();
			if (0 == ret) game->reinit();
		}

		delete game;

		return 0;
	}
	catch (BulletMLError be) {
		displayBulletMLError(be.what());

		std::cerr << "BulletML Error occured" << std::endl;
		std::cerr << be.what() << std::endl;
		delete game;
		return 1;
	}
	catch (std::runtime_error re) {
		std::cerr << re.what() << "\n";
		std::cerr << "SDL: " << SDL_GetError() << "\n";
		delete game;
		return 1;
	}
	catch (...) {
		std::cerr << "unknown exception.\n";
		delete game;
		return 1;
	}
}
