#include "game.h"
#include "command.h"
#include "player.h"
#include "bulletinfo.h"
#include "replay.h"
#include "conf.h"

#include <bulletml/bulletmlparser.h>
#include <bulletml/bulletmlparser-tinyxml.h>

#include <fstream>
#include <sstream>

EnemyBehavior::EnemyBehavior(const std::string& filename)
    : commandType_(Random)
{
    const Conf* conf = Conf::instance();
    if (conf->gameType() == Conf::XML) {
		BulletMLParser* bs = new BulletMLParserTinyXML(filename);
		bs->build();
		commands_.push_back(bs);
    }
    else if (conf->gameType() == Conf::ACT) {
		std::ifstream is(filename.c_str());
		check(is.is_open(), filename + "action File not Found!\n");
		parseActionFile(is);
    }
	else {
		const std::vector<BulletDescription*>& descs =
			Game::instance()->getBulletInfo()->descriptions();
		// vO}Ȃ̂ 1 n܂Ă̂́A
		// C[he߂肵܂B
		for (size_t i = 1; i < descs.size(); i++) {
			if (!descs[i]->isUserDefined()) {
				BulletMLParser* bs =
					new BulletMLParserTinyXML(descs[i]->xmlFile());
				bs->build();
				commands_.push_back(bs);
				init();
			}
		}
	}
}

void EnemyBehavior::parseActionFile(std::istream& is) {
    static const int MAXBUF = 256;
    char line[MAXBUF];

    while (!is.eof()) {
		is.getline(line, MAXBUF);
		std::istringstream iss(line);
		std::string command;
		iss >> command;

		ignoreBrankAndComment(command);
		if (command.empty()) continue;

		if (command == "Commands") {
			std::string args;
			iss >> args;
			if (args == "List") commandType_ = List;
			else if (args == "Random") commandType_ = Random;
			else check(0, "Unknown \"Commands\" argument" + args);
		}
		else {
			BulletMLParser* bs = new BulletMLParserTinyXML(command);
			bs->build();
			commands_.push_back(bs);
		}
    }

    init();
}

void EnemyBehavior::init() {
    commandIte_ = commands_.begin();
}

BulletMLParser* EnemyBehavior::getRandomAction() {
#ifndef ZAURUS
	if (Conf::instance()->isReplay() && commands_.size() != 1) {
		num_ = rnd(Game::instance()->getReplay()->bullets());
	}
	else {
#endif
		num_ = rnd(static_cast<int>(commands_.size()));
#ifndef ZAURUS
	}
#endif
    return commands_[num_];
}

BulletMLParser* EnemyBehavior::getNextAction() {
    BulletMLParser* ech = *commandIte_;
    commandIte_++;
    if (commandIte_ == commands_.end()) commandIte_ = commands_.begin();
    return ech;
}


int EnemyBehavior::getBulletNumber() {
	return num_+1;
}

Enemy::Enemy (const Point_& pnt, Surface* graph)
	: Charactor(pnt, graph), blue_(false)
{
}

BossEnemy::BossEnemy(const Point_& pnt, Surface* graph,
					 EnemyBehavior* behavior)
	: Enemy(pnt, graph), behavior_(behavior), firstPnt_(pnt_)
{
    behavior_->init();
	waitTime_ = Game::instance()->getFrameTurn() + 60;
	zako_ = false;
}

Enemy::Enemy (const Point_& pnt, const Point_& spd, Surface* graph,
			  BulletMLState* state, bool blue)
    : Charactor(pnt, graph), blue_(blue)
{
	com_.reset(new EnemyCommand(state, this));

    spd_ = spd;
    zako_ = true;
}

Enemy::~Enemy() {}

void Enemy::huntEvent() {
    if (com_.get() != 0) {
		com_->run();
		if (com_->isEnd()) {
			com_.reset(0);
			endBullet();
		}
    }
	else {
		nextBullet();
	}
}

void BossEnemy::endBullet() {
	waitTime_ = Game::instance()->getFrameTurn() + 60;
}

void BossEnemy::nextBullet() {
	if (waitTime_ <= Game::instance()->getFrameTurn()) {
		BulletMLParser* bulletml;
		if (behavior_->isRandomAction()) {
			bulletml = behavior_->getRandomAction();
			if (Conf::instance()->isNormalMode()) {
				Game::instance()->noticeChangeBullet(
					behavior_->getBulletNumber());
			}
		}
		else
			bulletml = behavior_->getNextAction();

		com_.reset(new EnemyCommand(bulletml, this));
	}
	else {
		pnt_ = firstPnt_;
/*
		Point vec(firstPnt_ - pnt_);
		if (vec.length() > 3) {
			vec.unit();
			pnt_ += vec * 10;
		}
		else {
			pnt_ = firstPnt_;
		}
*/
	}
}

void Enemy::hitByPlayer (Player*) {
}

void Enemy::hitByEnemy (Enemy*) {
}

void Enemy::hitByShot (Shot*) {
}

EnemyMgr::EnemyMgr() {}

void EnemyMgr::init() {
    behavior_.reset(new EnemyBehavior(Conf::instance()->actionFile()));
}

EnemyBehavior* EnemyMgr::getBehavior() {
	return behavior_.get();
}

