#ifndef BOHA_ENEMY
#define BOHA_ENEMY

#include "util.h"
#include "char.h"

#include <vector>

#include <SDL.h>

#include <boost/smart_ptr.hpp>

class EnemyCommand;
class BulletMLState;
class BulletMLParser;

class EnemyBehavior {
public:
    explicit EnemyBehavior(std::istream& is);
    explicit EnemyBehavior(const std::string& filename);

    BulletMLParser* getRandomAction();
    BulletMLParser* getNextAction();
    bool isRandomAction() const {
		return commandType_ == Random;
    }
    bool isListAction() const {
		return commandType_ == List;
    }

    void init();

	int getBulletNumber();

private:
    void parseActionFile(std::istream& is);

    typedef std::vector<BulletMLParser*> Commands;
    Commands commands_;
    Commands::iterator commandIte_;
    enum {List, Random} commandType_;

	int num_;
};

/// G̃NX
class Enemy : public Charactor {
public:
    Enemy (const Point_& pnt, const Point_& spd, Surface* graph,
		   BulletMLState* state, bool blue);
    virtual ~Enemy();

    void huntEvent ();

    virtual void hit (Charactor* bc) { bc->hitByEnemy(this); }
    virtual void hitByPlayer (Player* bc);
    virtual void hitByEnemy (Enemy* bc);
    virtual void hitByShot (Shot* bc);

	bool blue() const { return blue_; }

protected:
    Enemy(const Point_& pnt, Surface* graph);

private:
	virtual void endBullet() {}
	virtual void nextBullet() {}

protected:
    std::auto_ptr<EnemyCommand> com_;

	BulletMLState* state_;

	bool blue_;

    // R}h̎ۂ̑쌠͈ϏĂ܂
    friend class EnemyCommand;
};

/// Boss
class BossEnemy : public Enemy {
public:
    BossEnemy(const Point_& pnt, Surface* graph,
			  EnemyBehavior* behavior);

private:
	virtual void endBullet();
	virtual void nextBullet();

private:
    EnemyBehavior* behavior_;
	Point_ firstPnt_;
	int waitTime_;

};

class EnemyMgr {
public:
    EnemyMgr();

    void init();

    EnemyBehavior* getBehavior();

protected:
    // GĈƂɂgȂ
    std::auto_ptr<EnemyBehavior> behavior_;
};

#endif // ! BOHA_ENEMY
