#ifndef BOHA_GAME
#define BOHA_GAME

#include "singleton.h"
#include "mysdl_padinput.h"
#include "enemy.h"
#include "fpstimer.h"

#include <list>
#include <vector>
#include <queue>
#include <sstream>

#include <SDL.h>

#ifndef ZAURUS
# include <SDL_kanji/SDL_kanji.h>
#endif

#include <boost/smart_ptr.hpp>

class Screen;
class Surface;
class Charactor;
class Player;
class Enemy;
class Shot;
class EnemyCommand;
class BulletMLState;
class BulletInfo;
class BulletDescription;
class Select;
class Replay;
class CpuInputBase;

class Game : public Singleton<Game> {
private:
#ifndef ZAURUS
	typedef double Num_;
	typedef Point Point_;
#else
	typedef Num Num_;
	typedef NPoint Point_;
#endif

public:
    Game();

    // ԂI
    int title();
    void init();
    void reinit();
    void enemyInit();
    int start();

    Num_ spf() const { return spf_; }
    int shotNum() const { return shot_.size(); }
	int getFrameTurn() const;
	int getTurn() const;

    ~Game();

    const Player* player() const { return player_; }

    void addShot(const Point_& pnt, const Point_& spd, const Enemy* enemy);
    void addEnemy(const Point_& pnt, const Point_& spd, BulletMLState* state,
				  const Enemy* enemy);
    bool isStart() const { return start_; }
    bool isEnd() const { return end_; }
    bool isQuit() const { return quit_; }
    void end() { end_ = true; }
    void quit() { quit_ = true; }

    Num_ getRank() const { return rank_; }

    void getEnemyPosAndSpd(std::vector<std::pair<Point, Point> >& out) const;

	Screen* screen() { return screen_; }
	Surface* mainFrame();

	bool screenIsLarge() const;

#ifndef ZAURUS
	Kanji_Font* getFont() { return font_; }
#endif

	int waitAnyInput();
	int getAnyInput();
	void waitInputRelease();

	void endEv();

	void switchScreenMode(bool large);

	BulletInfo* getBulletInfo() { return bulletInfo_.get(); }

	void noticeChangeBullet(int index);

	int getLimit() const {
		return static_cast<int>((rank_+enemyNum_-1) * 100);
	}

#ifndef ZAURUS
	Replay* getReplay() { return replay_.get(); }
	void saveReplay(const std::string& c);
	void loadReplay(const std::string& prefix);
#endif

	void setNowBullet(BulletDescription* desc) { nowBullet_ = desc; }

	const std::list<Shot*> getShots() const { return shot_; }
	const std::list<Enemy*> getEnemys() const { return enemy_; }

private:
    void timerEv();
    void playerEv();
    void enemyEv();
    void hitEv();
    void updateScreen();
    void clearScreen();
    void deadEv();
	void levelEv();
	void charMoveValue();
	void charWallEvent();
	void charMoveGraphic();

	void processRanking();

    Screen* screen_;
	std::auto_ptr<Surface> mainFrame_;

    Player* player_;
    std::list<Enemy*> enemy_;
    std::list<Shot*> shot_;
    std::list<Charactor*> chars_;

	Surface* playerg_;
    Surface *enemygBoss_, *enemygZako_, *enemygBullet_;
    Surface *shotg_, *shotg2_;

	Num_ spf_;

    bool start_;
    bool end_;
	bool quit_;

    Num_ rank_;
	int enemyNum_;

    std::auto_ptr<MSDL::InputBase> inputMgr_;
#ifndef ZAURUS
	std::auto_ptr<CpuInputBase> cpuInput_;
#endif

    EnemyMgr enemyMgr_;

#ifndef ZAURUS
	Kanji_Font* font_;
#endif

	std::auto_ptr<MSDL::JoystickMgr> joystickMgr_;

	std::auto_ptr<BulletInfo> bulletInfo_;
	BulletDescription* nowBullet_;

	FPSTimer timer_;
	static const Num_ framePerSecond_;

	std::auto_ptr<Select> select_;

	int bulletInfoTicks_;
	int ellapsedSecOrRank_;

#ifndef ZAURUS
	std::auto_ptr<Replay> replay_;
#endif

	int lastEnemyTurn_;

};

#endif // BOHA_GAME
