#ifndef BOHA_WIN
#define BOHA_WIN

#include "config.h"
#include "util.h"
#include "singleton.h"
#include "config.h"

#include <SDL.h>

#include <cassert>

#include <string>
#include <iostream>
#include <utility>
#include <memory>

#ifdef ZAURUS
inline Uint32 convert_point(Uint32 val) {
	return (val >> 2) * 3;
}

inline Rect convert_rect(const Rect& rect) {
	return Rect(convert_point(rect.x()), convert_point(rect.y()),
				convert_point(rect.w()), convert_point(rect.h()), true);
}
#endif

// Surface NX
class Surface {
private:
#ifndef ZAURUS
	typedef Point Point_;
#else
	typedef NPoint Point_;
#endif

public:
	explicit Surface(SDL_Surface* surf) : surf_(surf) {}
    explicit Surface(const std::string& str);
	Surface(const Surface& rhs);
	Surface(const Point& size, int bpp, Uint32 flag,
		   Uint32 Rmask =0, Uint32 Gmask =0, Uint32 Bmask =0, Uint32 Amask =0);
    Surface(int x, int y, int bpp, Uint32 flag,
		   Uint32 Rmask =0, Uint32 Gmask =0, Uint32 Bmask =0, Uint32 Amask =0);
	///  Surface B
	Surface(Surface& src, const Rect& rect);

    virtual ~Surface();

    void fillRect(const Rect& rect, int color) {
#ifdef ZAURUS
		int ret = SDL_FillRect(surf_,
							   const_cast<SDL_Rect*>(convert_rect(rect).val()),
							   color);
#else
		int ret = SDL_FillRect(surf_, const_cast<SDL_Rect*>(rect.val()),
							   color);
#endif
		check(ret == 0, "Surface::fillRect");
    }
	void fillRect(const Rect& rect, Color col) {
		fillRect(rect, mapRGB(col));
	}
	void fill(int color) {
		int ret = SDL_FillRect(surf_, 0, color);
		check(ret == 0, "Surface::fillRect");
	}
	void fill(Color col) {
		fill(mapRGB(col));
	}

    void blit(Surface* src) {
		int ret = SDL_BlitSurface(src->surf_, 0, surf_, 0);
		check(ret == 0, "Surface::blit");
    }
    void blit(Surface* src, const Rect& srcrect, const Rect& dstrect) {
#ifdef ZAURUS
		int ret =
			SDL_BlitSurface(src->surf_, const_cast<SDL_Rect*>(
								convert_rect(srcrect).val()),
							surf_, const_cast<SDL_Rect*>(
								convert_rect(dstrect).val()));
#else
		int ret =
			SDL_BlitSurface(src->surf_, const_cast<SDL_Rect*>(srcrect.val()),
							surf_, const_cast<SDL_Rect*>(dstrect.val()));
#endif
		check(ret == 0, "Surface::blit");
    }
	void blit(Surface* src, const Point_& pnt) {
		SDL_Rect dst;
		dst.x = convert_point(pnt.xi());
		dst.y = convert_point(pnt.yi());
		int ret =
			SDL_BlitSurface(src->surf_, 0, surf_, &dst);
		check(ret == 0, "Surface::blit");
	}

    void printInformation(std::ostream& os) const;

#ifndef ZAURUS
    virtual const Point& size() const { return size_; }
#else
    virtual const NPoint& size() const { return size_; }
#endif

    void save(const std::string& filename) {
		SDL_SaveBMP(surf_, filename.c_str());
    }

	void setAlpha(Uint32 flag, Uint8 alpha) {
		int ret = SDL_SetAlpha(surf_, flag, alpha);
		check(ret == 0, "Surface::setAlpha");
	}

	void convertDisplayFormat();

    void setColorKey(Uint32 flag, Uint32 key) {
		int ret = SDL_SetColorKey(surf_, flag, key);
		check(ret == 0, "SurfacesetColorKey");
    }
	void setColorKey(Uint32 flag, Color col) {
		setColorKey(flag, mapRGB(col));
	}

	Num halfDiagonal() const { return halfDiagonal_; }
	const NPoint& halfSize() const { return halfSize_; }

	Uint32 getPixel(int x, int y);
	void putPixel(int x, int y, Uint32 pixel);

    Uint32 mapRGB(Color col) const {
		return col.map(surf_->format);
    }
	Color getRGB(Uint32 pixel) const {
		Uint8 r, g, b;
		SDL_GetRGB(pixel, surf_->format, &r, &g, &b);
		return Color(r, g, b);
	}

	void lock() {
		SDL_LockSurface(surf_);
	}
	void unlock() {
		SDL_UnlockSurface(surf_);
	}
	void update() {
		SDL_UpdateRect(surf_, 0, 0, 0, 0);
	}

#ifndef ZAURUS
    void putString(const std::string& str, const Point& pnt,
				   const Color& col = Color(0,0,0));
	int fontHeight();
	int fontWidth();
	int fontWidth(const std::string& str);

	// sge pN̂
	void drawHLine(Sint16 x1, Sint16 x2, Sint16 y, Color col);
	void drawVLine(Sint16 x, Sint16 y1, Sint16 y2, Color col);
	void drawLine(Sint16 x1, Sint16 y1, Sint16 x2, Sint16 y2, Color col);
	void drawRect(Sint16 x1, Sint16 y1, Sint16 x2, Sint16 y2, Color col);
	void drawRect(const Rect& r, Color col) {
		drawRect(r.x(), r.y(), r.x2(), r.y2(), col);
	}
#endif

	SDL_Surface* getSurface() { return surf_; }

protected:
	void calcCache();

protected:
	Surface() : surf_(0) {}

    SDL_Surface* surf_;

#ifndef ZAURUS
	double halfDiagonal_;

    Point size_;
	Point halfSize_;
#else
	Num halfDiagonal_;

    NPoint size_;
	NPoint halfSize_;
#endif
};

#endif // BOHA_WIN
