/**
 * $Id$
 *
 * Copyright (C) shinichiro.h <g940455@mail.ecc.u-tokyo.ac.jp>
 *  http://user.ecc.u-tokyo.ac.jp/~g940455/wp/
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "angle.h"

#include "util.h"

#ifdef ZAURUS

Num_ sintbl[DIV];
Num_ costbl[DIV];
int atantbl[DIV+1];
Num_ sqrttbl[DIV];

void initTable() {
#if 1
	FILE* fp = fopen("/opt/QtPalmtop/share/sdmkun/" "data/tables.dat", "rb");
	for (int i = 0; i < DIV; i++) {
		short v = fgetc(fp) << 8;
		v |= fgetc(fp);
		sintbl[i].val = v;
	}
	for (int i = 0; i < DIV; i++) {
		short v = fgetc(fp) << 8;
		v |= fgetc(fp);
		costbl[i].val = v;
	}
	for (int i = 0; i < DIV+1; i++) {
		short v = fgetc(fp) << 8;
		v |= fgetc(fp);
		atantbl[i] = v;
	}
	for (int i = 0; i < DIV; i++) {
		unsigned short v = fgetc(fp) << 8;
		v |= fgetc(fp);
		sqrttbl[i].val = v;
	}
	fclose(fp);
#else  // too slow
	for (int i = 0; i < DIV; i++) {
		sintbl[i] = Num_(std::sin(PI * 2 * i / DIV));
		costbl[i] = Num_(std::cos(PI * 2 * i / DIV));
		atantbl[i] = Num_(std::atan(1.0*i/DIV) * DIV / 2 / PI);
		sqrttbl[i] = Num_(std::sqrt(i));
	}
	atantbl[DIV] = Num_(DIV/8);
#endif
}

#endif

#ifndef ZAURUS
Num_ dtor(Num_ x) { return x*PI/180; }
Num_ rtod(Num_ x) { return x*180/PI; }
#else
Num_ dtor(const Num_ x) {
	static const Num_ r(4096.0/360.0);
	Num_ ret;
	ret.val = x * r;
	return ret;
}
Num_ rtod(const Num_ x) {
	static const Num_ r(360.0/4096.0);
	return x.val * r;
}
#endif

#ifdef ZAURUS
Num_ square(Num_ n) {
	if (n == 0) return Num_(0);

	int m = 0;

	while (n < 64) {
		n.val <<= 6;
		m += 3;
	}
	while (n >= DIV) {
		n.val >>= 6;
		m -= 3;
	}

	Num_ ret(sqrttbl[n]);
	if (m > 0) ret.val >>= m;
	else if (m < 0) ret.val <<= (-m);

	return ret;
}

Num_ getAngle(Num_ nx, Num_ ny) {
	int tx, ty;
    int f, od, tn;
	int x = nx.val;
	int y = ny.val;

    if ( x==0 && y==0 ) {
		return(Num_(0));
    }

    if ( x < 0 ) {
		tx = -x;
		if ( y < 0 ) {
			ty = -y;
			if ( tx > ty ) {
				f = 1;
				od = DIV*3/4; tn = ty*DIV/tx;
			} else {
				f = -1;
				od = DIV; tn = tx*DIV/ty;
			}
		} else {
			ty = y;
			if ( tx > ty ) {
				f = -1;
				od = DIV*3/4; tn=ty*DIV/tx;
			} else {
				f=1;
				od = DIV/2; tn=tx*DIV/ty;
			}
		}
    } else {
		tx = x;
		if ( y < 0 ) {
			ty = -y;
			if ( tx > ty ) {
				f = -1;
				od = DIV/4; tn = ty*DIV/tx;
			} else {
				f = 1;
				od = 0; tn = tx*DIV/ty;
			}
		} else {
			ty = y;
			if ( tx > ty ) {
				f = 1;
				od = DIV/4; tn = ty*DIV/tx;
			} else {
				f = -1;
				od = DIV/2; tn = tx*DIV/ty;
			}
		}
    }
	Num_ ret;
	ret.val = od+atantbl[tn]*f;
    return ret;
}
#endif


